#!/bin/bash
# This script will whitelist this node with an Open Attestation Service
#
# TODO: should do some basic input validation so no errors are produced 
# when entering hardware information into the DB on the OAT Server.
if [ $UID -ne 0 ]; then
    echo "This can only be executed as root.  Aborting."
    exit 1
fi

if [ $# -ne 2 ]; then 
    echo "Usage: $0 <oat_server_fqdn> <my_ip>"
    echo "Where <oat_server_fqdn> must be the hostname of the OAT server node,"
    echo "and <my_ip> is the IP of the interface the OAT service will contact"
    echo "the OAT Client on this node (might be a private IP)."
    exit 1
fi

# OAT Server FQDN
oat_svc_host=$1

# The IP we'll be contacting the OAT server *from* (could be a private IP)
ipaddr=$2

# Our hostname (just to be sure) 
hostname=`hostname -f`

# Get some details of the system (hardware, OS, etc.) to enter into the OAT db
oem_manu=`dmidecode -s system-manufacturer`
oem_desc=`dmidecode -s system-product-name`
os=`cat /etc/redhat-release | awk -F"release" '{print $1}'`
os_ver=`cat /etc/redhat-release | awk -F"release" '{print $2}'`
os_desc=$os
vmm=`virsh version | grep -i hyper | awk '{print $3}'`
vmm_ver=`virsh version | grep -i hyper | awk '{print $4}'`-$hostname
vmm_desc=$vmm
bios=`dmidecode -s bios-vendor`
bios_ver=`dmidecode -s bios-version`
bios_desc=`dmidecode -s baseboard-product-name`
pcr_00=`cat \`find /sys -name pcrs\` | grep PCR-00 | cut -c 8-80 | perl -pe 's/ //g'`
pcr_18=`cat \`find /sys -name pcrs\` | grep PCR-18 | cut -c 8-80 | perl -pe 's/ //g'`

# echo \'$oat_svc_host\' \'$hostname\' \'$ipaddr\' \'$oem_manu\' \'$oem_desc\' \'$os\' \'$os_ver\' \'$os_desc\' \'$vmm\' \'$vmm_ver\' \'$vmm_desc\' \'$pcr_18\' \'$bios\' \'$bios_ver\' \'$bios_desc\' \'$pcr_00\'

# set oat service SSL cert path
export OAT_CERTFILE=$HOME/.oat_cer

# get oat service SSL cert
oat_cert -h $oat_svc_host 

# Enter the system hardware manufacturer (OEM) into the oat_db
str="`oat_view_oem -h $oat_svc_host`"
str2="`echo $str | awk -F\] '{print $1}' | awk -F\[ '{print $2}'`"
if [ "" == "$str2" -o "$str" != "${str/\"Name\":\"$oem_manu\"/}" ];then
  oat_oem -a -h $oat_svc_host "{\"Name\":\"$oem_manu\",\"Description\":\"$oem_desc\"}"
else
  echo -e "\"Name\":\"$oem_manu\" is  already added ...\n"
fi

# Enter the operating sytem into the oat_db (RH-like systems only)
str="`oat_view_os -h $oat_svc_host`"
str2="`echo $str | awk -F\] '{print $1}' | awk -F\[ '{print $2}'`"
if [ "" == "$str2" -o  $str != "${str/\"Name\":\"$os\",\"Version\":\"$os_ver\"/}" ];then
  oat_os -a -h $oat_svc_host "{\"Name\":\"$os\",\"Version\":\"$os_ver\",\"Description\":\"$os_desc\"}"
else
  echo -e "\"Name\":\"$os\",\"Version\":\"$os_ver\" is already added ...\n"
fi

# Enter VMM measured launch environment (mle) into the oat_db
str=`oat_view_mle -h $oat_svc_host "{\"mleName\":\"$vmm\",\"mleVersion\":\"$vmm_ver\",\"osName\":\"$os\",\"osVersion\":\"$os_ver\"}"`
str2="`echo $str | awk -F, '{print $1}' | awk -F: '{print $2}'`"
if [ "null" == "$str2" -o $str != "${str/\"Name\":\"$vmm\",\"Version\":\"$vmm_ver\"/}" ];then
  oat_mle -a -h $oat_svc_host "{\"Name\":\"$vmm\",\"Version\":\"$vmm_ver\",\"OsName\":\"$os\",\"OsVersion\":\"$os_ver\",\"Attestation_Type\":\"PCR\",\"MLE_Type\":\"VMM\",\"Description\":\"$vmm_desc\",\"MLE_Manifests\":[{\"Name\":\"18\",\"Value\":\"$pcr_18\"}]}"
else
  echo  -e "\"Name\":\"$vmm\",\"Version\":\"$vmm_ver\",\"OsName\":\"$os\",\"OsVersion\":\"$os_ver\"  is already added ...\n"
fi

# Enter BIOS managed launch environment (mle) into the oat_db
str=`oat_view_mle -h $oat_svc_host "{\"mleName\":\"$bios\",\"mleVersion\":\"$bios_ver\",\"oemName\":\"$oem_manu\"}"`
str2="`echo $str | awk -F, '{print $1}' | awk -F: '{print $2}'`"
if [ "null" == "$str2" -o "$str" != "${str/\"Name\":\"$bios\",\"Version\":\"$bios_ver\"/}" ];then
  oat_mle -a -h $oat_svc_host "{\"Name\":\"$bios\",\"Version\":\"$bios_ver\",\"OemName\":\"$oem_manu\",\"Attestation_Type\": \"PCR\",\"MLE_Type\":\"BIOS\",\"Description\":\"$bios_desc\",\"MLE_Manifests\":[{\"Name\":\"0\",\"Value\":\"$pcr_00\"}]}"
else
  echo -e  "\"Name\":\"$bios\",\"Version\":\"$bios_ver\",\"OemName\":\"$oem_manu\"  is already added ...\n"
fi

# add the host to the database
str=`oat_host  -s -h $oat_svc_host "{$hostname}"`
str2="`echo $str | awk -F] '{print $1}' | awk -F[ '{print $2}'`"
if [ "" == "$str2" -o $str != "${str/\"HostName\":\"$hostname\"/}" ];then
  oat_host -a -h $oat_svc_host "{\"HostName\":\"$hostname\",\"IPAddress\":\"$ipaddr\",\"Port\":\"9999\",\"BIOS_Name\":\"$bios\",\"BIOS_Version\":\"$bios_ver\",\"BIOS_Oem\":\"$oem_manu\",\"VMM_Name\":\"$vmm\",\"VMM_Version\":\"$vmm_ver\",\"VMM_OSName\":\"$os\",\"VMM_OSVersion\":\"$os_ver\",\"Email\":\"\",\"AddOn_Connection_String\":\"\",\"Description\":\"\"}"
else
  echo  -e "\"HostName\":\"$hostname\"  is already added ...\n"
fi

# attest the host
oat_pollhosts -h $oat_svc_host "{\"hosts\":[\"$hostname\"]}" | grep '"trust_lvl":"trusted"' &> /dev/null

if [ $? -eq 0 ]; then \
       echo "Node Attestation Successful!"
       exit 0
else
       echo "Node Attestation Failed!"
       exit 1
fi

